/*
 * Licensed to the Apache Software Foundation (ASF) under one or more contributor license
 * agreements. See the NOTICE file distributed with this work for additional information regarding
 * copyright ownership. The ASF licenses this file to You under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apache.geode.cache;

import org.apache.geode.cache.control.RebalanceOperation;

/**
 * Thrown when a {@link RebalanceOperation} occurs concurrently with a transaction. This can be
 * thrown while doing transactional operations or during commit.
 *
 * <p>
 * This exception only occurs when a transaction involves partitioned regions.
 *
 * @since GemFire 6.6
 */
public class TransactionDataRebalancedException extends TransactionException {

  private static final long serialVersionUID = -2217135580436381984L;

  public TransactionDataRebalancedException(String s) {
    super(s);
  }

  public TransactionDataRebalancedException(Throwable cause) {
    super(cause);
  }

  public TransactionDataRebalancedException(String message, Throwable cause) {
    super(message, cause);
  }
}
