/*
 * Licensed to the Apache Software Foundation (ASF) under one or more contributor license
 * agreements. See the NOTICE file distributed with this work for additional information regarding
 * copyright ownership. The ASF licenses this file to You under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apache.geode.internal.cache.tier.sockets.command;

import java.io.IOException;

import org.apache.geode.annotations.Immutable;
import org.apache.geode.internal.cache.tier.Command;
import org.apache.geode.internal.cache.tier.sockets.Message;
import org.apache.geode.internal.cache.tier.sockets.Part;

/**
 * Adds a callbackArg to PutAll80
 *
 */
public class PutAllWithCallback extends PutAll80 {

  @Immutable
  private static final PutAllWithCallback singleton = new PutAllWithCallback();

  public static Command getCommand() {
    return singleton;
  }

  protected PutAllWithCallback() {}

  @Override
  protected String putAllClassName() {
    return "putAllWithCallback";
  }

  @Override
  protected Object getOptionalCallbackArg(Message msg) throws ClassNotFoundException, IOException {
    Part callbackPart = msg.getPart(5);
    return callbackPart.getObject();
  }

  @Override
  protected int getBasePartCount() {
    return 6;
  }
}
