/*
 * Licensed to the Apache Software Foundation (ASF) under one or more contributor license
 * agreements. See the NOTICE file distributed with this work for additional information regarding
 * copyright ownership. The ASF licenses this file to You under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package org.apache.geode.internal.util;

import static org.assertj.core.api.Assertions.assertThat;

import org.junit.Test;
import org.junit.experimental.categories.Category;

import org.apache.geode.test.junit.categories.SecurityTest;

@Category({SecurityTest.class})
public class PasswordUtilJUnitTest {

  /**
   * Tests PasswordUtil's decrypt method (which is now deprecated) and verifies that backwards
   * compatibility has not been broken with the removal of PasswordUtil's encryption methods.
   * Previously encrypted passwords may still be used and decrypted.
   */
  @Test
  public void testPasswordUtil() {
    String password = "password";
    String encrypted = "encrypted(C3CDC3485F7FF64381841CD344CBDF8A)";
    String decrypted = PasswordUtil.decrypt(encrypted);
    assertThat(decrypted).isEqualTo(password);
  }

  @Test
  public void testNullPassword() throws Exception {
    assertThat(PasswordUtil.decrypt(null)).isNull();
  }
}
